import numpy as np
import pandas as pd
import geopy.distance
import gauss_sig  as gs
import predictionn as pred

'''
air_lat = 42.6537
air_lon = 23.3829
air_elv = 595
c = (air_lat, air_lon, air_elv)
'''
c1 = (42.732292, 23.310972, 0)
c2 = (42.680558, 23.296786, 0)
c3 = (42.666508, 23.400164, 0)
c4 = (42.669797, 23.268403, 0)
#Takes in deg as format xx-xx-xx.xxN/E/S/W
def convert(tude):
	multiplier = 1 if tude[-1] in ['N', 'E'] else -1
	return multiplier * sum(float(x) / 60 ** n for n, x in enumerate(tude[:-1].split('-')))

#Takes deg, cleans it, and converts to decimal
def deg_to_decimal(data):
	coords = []
	for i in range(len(data)):
		coords.append(convert(data.iat[i,0].replace("°","-").replace("\"", "").replace("\'\'","-") .replace("\'","")))
	return coords

#Takes in 2 coordinates tuples and returns the distance in meters
#This uses geopy library for more accuracy since it uses an eleptical model
def distance(c1,c2):
	return geopy.distance.distance(c1,c2).m

#Location of files used
industrial = pd.read_csv(r"data/industrial_pollution.csv", encoding="latin_1")
sofia_topo = pd.read_csv(r"data/sofia_topo.csv")
weatherData= pd.read_csv(r"data/weather_lbsf_20161101-20161130_IP_train.csv") 

weatherData['Date'] = pd.to_datetime(weatherData[['year','Month','day']])
weatherData.drop(['year','Month','day','PRCPMAX','PRCPMIN'],axis=1,inplace=True)

#Cleaning industrial data and converting to decimal float
x_ind = deg_to_decimal(industrial[["X*"]])
y_ind = deg_to_decimal(industrial[["Y*"]])
s_height = industrial[["m"]].values.flatten()
g_ind = industrial[["t/y"]].values.flatten()

wind_speed=weatherData[["sfcWindAVG"]].values.flatten()
wind_day=weatherData[["sfcWindAVG"]].values.flatten()

'''
x_sofia = sofia_topo[["Lat"]].values.flatten()
y_sofia = sofia_topo[["Lon"]].values.flatten()
z_sofia = sofia_topo[["Elev"]].values.flatten()

z = []

for i in range(len(x_ind)):
	for j in range(len(x_sofia)):
		if(((str(x_ind[i])[0:5]) in (str(x_sofia[j]))) and ((str(y_ind[i])[0:5]) in str(y_sofia[j]))):
			z.append([z_sofia[j],i])
'''

result = {}
result["Date"] = []
result["Station1"] = []
result["Station2"] = []
result["Station3"] = []
result["Station4"] = []

for i in range(len(x_ind)):
	coord = (x_ind[i], y_ind[i], s_height[i])
	q = g_ind[i] * 1000 * 1000 / 190 / 24 / 60 / 60 # in g/s
	h = s_height[i] # in m
	d1 = distance(coord,c1) # in m
	d2 = distance(coord,c2) # in m
	d3 = distance(coord,c3) # in m
	d4 = distance(coord,c4) # in m
	stab = "B"
	for i in range(len(wind_speed)):
		u = wind_speed[i]
		result["Date"].append(weatherData['Date'][i])
		result["Station1"].append(gs.gauss_func(x_ind[i],y_ind[i],0,q,u,h,d1,stab))
		result["Station2"].append(gs.gauss_func(x_ind[i],y_ind[i],0,q,u,h,d2,stab))
		result["Station3"].append(gs.gauss_func(x_ind[i],y_ind[i],0,q,u,h,d3,stab))
		result["Station4"].append(gs.gauss_func(x_ind[i],y_ind[i],0,q,u,h,d4,stab))

#print("\nEntries: " + str(len(c)))

# structuredData = []
# for i in range(len(c)):
# 	date = weatherData['Date'][i]
# 	pollution = pd.Series(c[i])
# 	obj = {}
# 	obj["date"] = date
# 	obj["Station1"] = pollution["Station1"]
# 	obj["Station2"] = pollution["Station2"]
# 	obj["Station3"] = pollution["Station3"]
# 	obj["Station4"] = pollution["Station4"]

# 	structuredData.append(obj)
# 	break

test= pd.DataFrame(data=result)

#pred.predict(test)
test.to_csv('csv_example')
